// SDL_ph_image.c

#include <stdlib.h>
#include <Ph.h>
#include <photon/Pg.h>

#include "SDL_error.h"
#include "SDL_endian.h"
#include "SDL_ph_image_c.h"

#ifdef HAVE_KSTAT
#include <kstat.h>
#endif

/* Various screen update functions available */
static void ph_NormalUpdate(_THIS, int numrects, SDL_Rect *rects);
static void ph_DummyUpdate(_THIS, int numrects, SDL_Rect *rects);

int ph_SetupImage(_THIS, SDL_Surface *screen)
{
	int type = 0;

	/* Determine image type */
	switch(screen->format->BitsPerPixel)
	{
		case 8:{
			type = Pg_IMAGE_PALETTE_BYTE;
		}
		break;

		case 16:{
			type = Pg_IMAGE_DIRECT_565; 
		}
		break;

		case 24:{
			type = Pg_IMAGE_DIRECT_888;
		}
		break;

		default:{
		/* should never get here */
			fprintf(stderr,"error: unsupported bbp = %d\n",
					screen->format->BitsPerPixel);
			return -1;
		}
		break;
	}

	if ((SDL_Image = PhCreateImage( NULL, screen->w, screen->h, type, NULL, 0, 0 )) 
		== NULL)
	{
		fprintf(stderr,"error: PhCreateImage failed.\n");
		return -1;
	}
	if ((screen->pixels = (void *)malloc(screen->h*screen->pitch)) == NULL)
	{
       fprintf(stderr,"error: could not malloc memory for screen->pixels\n");
       return -1;
	}
	SDL_Image->image = (char *)screen->pixels;

	this->UpdateRects = ph_NormalUpdate;

	return 0;
}

void ph_DestroyImage(_THIS, SDL_Surface *screen)
{
	if (SDL_Image->image)
	{
		free(SDL_Image->image);
		SDL_Image->image = NULL;
	}

	PhReleaseImage(SDL_Image);

	if ( screen ) {
    	screen->pixels = NULL;
	}
}

int ph_ResizeImage(_THIS, SDL_Surface *screen, Uint32 flags)
{
    int retval;

    ph_DestroyImage(this, screen);
	/* No image when using GL */
    return ( flags & SDL_OPENGL ) ? 0 : ph_SetupImage(this, screen);
}

int ph_AllocHWSurface(_THIS, SDL_Surface *surface)
{
        return(-1);
}

void ph_FreeHWSurface(_THIS, SDL_Surface *surface)
{
        return;
}

int ph_FlipHWSurface(_THIS, SDL_Surface *surface)
{
        return(0);
}

int ph_LockHWSurface(_THIS, SDL_Surface *surface)
{
        if ( (surface == SDL_VideoSurface) && blit_queued ) {
//                XSync(GFX_Display, False);
				PgFlush();
                blit_queued = 0;
        }
        return(0);
}

void ph_UnlockHWSurface(_THIS, SDL_Surface *surface)
{
        return;
}

/* Byte-swap the pixels in the display image */
static void ph_SwapAllPixels(SDL_Surface *screen)
{
        int x, y;

        switch (screen->format->BytesPerPixel) {
            case 2: {
                Uint16 *spot;
                for ( y=0; y<screen->h; ++y ) {
                        spot = (Uint16 *) ((Uint8 *)screen->pixels +
                                                y * screen->pitch);
                        for ( x=0; x<screen->w; ++x, ++spot ) {
                                *spot = SDL_Swap16(*spot);
                        }
                }
            }
            break;

            case 4: {
                Uint32 *spot;
                for ( y=0; y<screen->h; ++y ) {
                        spot = (Uint32 *) ((Uint8 *)screen->pixels +
                                                y * screen->pitch);
                        for ( x=0; x<screen->w; ++x, ++spot ) {
                                *spot = SDL_Swap32(*spot);
                        }
                }
            }
            break;

            default:
                /* should never get here */
                break;
        }
}

static void ph_SwapPixels(SDL_Surface *screen, int numrects, SDL_Rect *rects)
{
        int i;
        int x, minx, maxx;
        int y, miny, maxy;

        switch (screen->format->BytesPerPixel) {
            case 2: {
                Uint16 *spot;
                for ( i=0; i<numrects; ++i ) {
                        minx = rects[i].x;
                        maxx = rects[i].x+rects[i].w;
                        miny = rects[i].y;
                        maxy = rects[i].y+rects[i].h;
                        for ( y=miny; y<maxy; ++y ) {
                                spot = (Uint16 *) ((Uint8 *)screen->pixels +
                                                y * screen->pitch + minx * 2);
                                for ( x=minx; x<maxx; ++x, ++spot ) {
                                        *spot = SDL_Swap16(*spot);
                                }
                        }
                }
            }
            break;

            case 4: {
                Uint32 *spot;
                for ( i=0; i<numrects; ++i ) {
                        minx = rects[i].x;
                        maxx = rects[i].x+rects[i].w;
                        miny = rects[i].y;
                        maxy = rects[i].y+rects[i].h;
                        for ( y=miny; y<maxy; ++y ) {
                                spot = (Uint32 *) ((Uint8 *)screen->pixels +
                                                y * screen->pitch + minx * 4);
                                for ( x=minx; x<maxx; ++x, ++spot ) {
                                        *spot = SDL_Swap32(*spot);
                                }
                        }
                }
            }
            break;

            default:
                /* should never get here */
                break;
        }
}

static PhPoint_t ph_pos;
static PhRect_t ph_rect;
static int i;

static void ph_NormalUpdate(_THIS, int numrects, SDL_Rect *rects)
{

    for ( i=0; i<numrects; ++i ) 
	{
    	if ( rects[i].w == 0 ) { /* Clipped? */
                continue;
        }

		ph_pos.x = rects[i].x;
		ph_pos.y = rects[i].y;
		ph_rect.ul.x = rects[i].x;
		ph_rect.ul.y = rects[i].y;
		ph_rect.lr.x = rects[i].x + rects[i].w;
		ph_rect.lr.y = rects[i].y + rects[i].h;

		if (PgDrawPhImageRectmx( &ph_pos, SDL_Image, &ph_rect, 0 ) < 0)
		{
			fprintf(stderr,"error: PgDrawPhImageRectmx failed.\n");
		}
	}
    if (PgFlush() < 0)
	{
    	fprintf(stderr,"error: PgFlush failed.\n");
    }
}
